package com.sun.jndi.dns;

import java.util.Date;
import java.lang.ref.SoftReference;

class ZoneNode extends NameNode
{
    private SoftReference<NameNode> contentsRef;
    private long serialNumber;
    private Date expiration;
    
    ZoneNode(String s) {
        super(s);
        this.contentsRef = null;
        this.serialNumber = -1L;
        this.expiration = null;
    }
    
    @Override
    protected NameNode newNameNode(String s) {
        return new ZoneNode(s);
    }
    
    synchronized void depopulate() {
        this.contentsRef = null;
        this.serialNumber = -1L;
    }
    
    synchronized boolean isPopulated() {
        return this.getContents() != null;
    }
    
    synchronized NameNode getContents() {
        return (this.contentsRef != null) ? ((NameNode)this.contentsRef.get()) : null;
    }
    
    synchronized boolean isExpired() {
        return this.expiration != null && this.expiration.before(new Date());
    }
    
    ZoneNode getDeepestPopulated(DnsName dnsName) {
        ZoneNode zoneNode = this.isPopulated() ? this : null;
        int i = 1;
        while (i < dnsName.size()) {
            this = (ZoneNode)this.get(dnsName.getKey(i));
            if (this == null) {
                break;
            }
            if (this.isPopulated()) {
                zoneNode = this;
            }
            ++i;
        }
        return zoneNode;
    }
    
    NameNode populate(DnsName dnsName, ResourceRecords resourceRecords) {
        NameNode referent = new NameNode(null);
        for (int i = 0; i < resourceRecords.answer.size(); ++i) {
            ResourceRecord resourceRecord = (ResourceRecord)resourceRecords.answer.elementAt(i);
            DnsName name = resourceRecord.getName();
            if (name.size() > dnsName.size() && name.startsWith(dnsName)) {
                NameNode add = referent.add(name, dnsName.size());
                if (resourceRecord.getType() == 2) {
                    add.setZoneCut(true);
                }
            }
        }
        ResourceRecord resourceRecord2 = (ResourceRecord)resourceRecords.answer.firstElement();
        synchronized (this) {
            this.contentsRef = new SoftReference(referent);
            this.serialNumber = getSerialNumber(resourceRecord2);
            this.setExpiration(getMinimumTtl(resourceRecord2));
            return referent;
        }
    }
    
    private void setExpiration(long n) {
        this.expiration = new Date(System.currentTimeMillis() + 1000L * n);
    }
    
    private static long getMinimumTtl(ResourceRecord resourceRecord) {
        String s = (String)resourceRecord.getRdata();
        return Long.parseLong(s.substring(s.lastIndexOf(32) + 1));
    }
    
    int compareSerialNumberTo(ResourceRecord resourceRecord) {
        return ResourceRecord.compareSerialNumbers(this.serialNumber, getSerialNumber(resourceRecord));
    }
    
    private static long getSerialNumber(ResourceRecord resourceRecord) {
        String s = (String)resourceRecord.getRdata();
        int n = s.length();
        int endIndex = -1;
        for (int i = 0; i < 5; ++i) {
            endIndex = n;
            n = s.lastIndexOf(32, endIndex - 1);
        }
        return Long.parseLong(s.substring(n + 1, endIndex));
    }
}
